#!/usr/bin/perl

use IO::Socket;

my $conf;
my %ip_to_handle;

my $ConfigFile = shift @ARGV or die "usage: monitor-cs.pl <config-file>";

$|=1;

load_config($ConfigFile);
run();
exit(0);


sub run
	{
	my %last = ();
	my %this;
	my ($h, $ip, $c_ip, $players, $err);

	while(1) {
   	   $players = 0;
   	   %this = check_cs();

   	   foreach $h (values %ip_to_handle) {
        	$ip   = $this{$h};
        	$c_ip = $last{$h};

        	if ($ip ne $c_ip) {
           	   if ($ip =~ /\d/) {
			$err = uplink_message($h,"Connected to $ip"); 
			}
           	   else {
			$err = uplink_message($h,"Disconnected"); 
			}
           	   }

        	if ($ip) { $players++;}
        	}

   	   %last = %this;

   	   if ($players) { sleep(30);}
   	   else 	 {sleep(120);}
   	   }
	}


sub check_cs
        {
        my ($k,$line,$src,$port,$dst,$sport,$dport,%count);
        my ($skey,$dkey,%map,$dev);
        my %total = ();

	$dev = $conf->{device};
	if ($dev !~ /^[a-z]{3}\d$/) {
	   print "Invalid device $dev\n";
	   exit;
	   }

        open(I,"./my_tcpdump $dev 2>/dev/null |");
        while($line = <I>) {
           chomp $line;
           if ($line =~ /(\d+\.\d+\.\d+\.\d+)\.(\d+)\s+>\s+(\d+\.\d+\.\d+\.\d+)\.(\d+)/) {
                $src = $1;
                $sport = $2;
                $dst = $3;
                $dport = $4;

                # Skip packets to/from privileged ports
                next if $sport < 1024 or $dport < 1024;

                # switch order if necessary so that local addr is "$src"
                ($src,$dst,$sport,$dport) = ($dst,$src,$dport,$sport) if exists $ip_to_handle{$dst};
                next if ! exists $ip_to_handle{$src};

                $skey = "$src:$sport";  # src IP:PORT
                $dkey = "$dst:$dport";  # dst IP:PORT

                $total{$skey}{$dkey}++;
                $count{$skey}++;
                }
           }
        close(I);

        # If more than half the packets went to a specific dest/port then we
        # select this as a candidate for the CS server.

        foreach $skey (keys %total) {
           foreach $dkey (keys %{$total{$skey}}) {

              # If we sent less than 5 packets then this isn't a server
              next if $total{$skey}{$dkey} < 5;

              if ($total{$skey}{$dkey} > $count{$skey}/2) {
                my ($src,$port) = split(/:/, $skey);
                my $h = $ip_to_handle{$src};

                # Confirm that this candidate is a CS server by sending
                # a "ping" request and receive a valid response.
                $p = do_ping($dkey);

                $map{$h} = $dkey if $p > 0;
                }
              }
           }


        return %map;
        }

sub uplink_message
        {
        my $handle = shift;
        my $msg = shift;
	my $host = "http://" . $conf->{server};
        my $url;
        my $sock;

	print "Uplink: $msg\n";

        # URL encode this message
        $msg =~ s/\s+/+/g;

        $url = "$host/util/runtag.live/$handle?tag=LOGGER&action=store&info=$msg";

        $sock = IO::Socket::INET->new(
                        PeerAddr => $conf->{server},
                        PeerPort => 80,
                        Proto    => 'tcp');

	if (! $sock) {
	   print "Could not open network connection\n";
	   return 0;
	   }

	# unbuffer
        select $sock; $|=1;
	select STDOUT;

        print $sock "GET $url\n";
        sleep(1);
        close($sock);
	return 0;
        }

# ----------------------------------------------------------------------------

my %server_hash;

sub do_ping
        {
        my $server = shift;
        my ($i,$p,$msg);

	return 1 if exists $server_hash{$server};

        for($i=0; $i<5; ++$i) {
           $msg = query_server($server,"ping");
           last if length($msg);
           }

        return "" if length($msg) != 6;
        ($p) =  unpack('S', substr($msg,4));

	$server_hash{$server} = 1 if $p;
        return $p;
        }

sub query_server
    {
    my ($server, $port) = split ':', shift;
    my $cmd = shift;
    my $msg = "";
    my $TIMEOUT = 2;
    my $MAXLEN  = 1024;

    my $sock = IO::Socket::INET->new (PeerAddr => $server,
         PeerPort => $port,
         Proto    => 'udp') or return "";

    $sock->send("\xFF"x4 . $cmd) or return "";

    # -- Alarm loop
    eval {
        local $SIG{ALRM} = sub { die "alarm time out" };
        alarm $TIMEOUT;

        #-- Receive msg from the server
        $sock->recv($msg, $MAXLEN)      or die "recv: $!";

        alarm 0;
        1;  # return value from eval on normalcy
    };

    close($sock);

    return $msg;
}

sub load_config
	{
	my $file = shift;
	my ($h,$ip,@args,$line,$cmd);
	local(*I);

	open(I,"<$file") or die "Missing config file \"$file\"";
	while($line = <I>) {
	   next if $line =~ /^\s*#/;
	   $line =~ s/#.*$//;

	   chomp $line;
	   next if ! $line;

	   ($cmd,@args) = split(/\s+/, $line);
	   $cmd = lc $cmd;

	   if ($cmd eq "device") {
		$conf->{device} = $args[0];
		print "Using device $args[0]\n";
		}
	   if ($cmd eq "map-handle") {
		my $h  = shift @args;
		my $ip;

		foreach $ip (@args) {
		   $ip_to_handle{$ip} = $h;
		   }
		}
	   if ($cmd eq "server") {
		$conf->{server} = $args[0];
		}
	   }
	close(I);

	if (! $conf->{device}) {
	   print "Missing \"device\" entry in config file $ConfigFile\n";
	   exit;
	   }

	if (! $conf->{server}) {
	   print "Missing 'server' entry in config file $ConfigFile\n";
	   exit;
	   }

	}
