#!/usr/bin/perl

use IO::Socket;

my $conf;
my %ip_to_handle;

my $ConfigFile = shift @ARGV or die "usage: update-ip.pl <config-file>";

$|=1;

load_config($ConfigFile);

uplink_message();

sub uplink_message
        {
	my $host = "http://" . $conf->{server};
        my $url;
        my $sock;

        $url = "$host/util/runtag.live/$handle?tag=LOGGER&action=sf_serverid";

        $sock = IO::Socket::INET->new(
                        PeerAddr => $conf->{server},
                        PeerPort => 80,
                        Proto    => 'tcp');

	if (! $sock) {
	   print "Could not open network connection\n";
	   return 0;
	   }

	# unbuffer
        select $sock; $|=1;
	select STDOUT;

        print $sock "GET $url\n";
        sleep(1);
        close($sock);
	return 0;
        }

# ----------------------------------------------------------------------------

sub load_config
	{
	my $file = shift;
	my ($h,$ip,@args,$line,$cmd);
	local(*I);

	open(I,"<$file") or die "Missing config file \"$file\"";
	while($line = <I>) {
	   next if $line =~ /^\s*#/;
	   $line =~ s/#.*$//;

	   chomp $line;
	   next if ! $line;

	   ($cmd,@args) = split(/\s+/, $line);
	   $cmd = lc $cmd;

	   if ($cmd eq "device") {
		$conf->{device} = $args[0];
		print "Using device $args[0]\n";
		}
	   if ($cmd eq "map-handle") {
		my $h  = shift @args;
		my $ip;

		foreach $ip (@args) {
		   $ip_to_handle{$ip} = $h;
		   }
		}
	   if ($cmd eq "server") {
		$conf->{server} = $args[0];
		}
	   }
	close(I);

	if (! $conf->{device}) {
	   print "Missing \"device\" entry in config file $ConfigFile\n";
	   exit;
	   }

	if (! $conf->{server}) {
	   print "Missing 'server' entry in config file $ConfigFile\n";
	   exit;
	   }

	}
